@@ -20,7 +20,7 @@ create_test_tmpdir
 # This wrapper touches a file to let us know when scan runs successfully.
 cat > "$test_tmpdir"/scan <<EOF
 #! /bin/sh
-HOME=
+HOME=/nonexistent
 echo "\$@" > "$test_tmpdir"/scan.args
 "$scan_binary" "\$@" > "$logdir/scan.\$\$.stdout" 2> "$logdir/scan.\$\$.stderr"
 status=\$?
@@ -49,7 +49,7 @@ The scan program should be called on this message.
 EOF
 
 start_test scan
-start_xlbiff_under_xvfb ${XLBIFF_DEBUG:+-debug} \
+start_xlbiff_under_xvfb ${XLBIFF_DEBUG:+-debug "$XLBIFF_DEBUG"} \
                         -file "$test_tmpdir/mailbox" -xrm "$xrm_scan"
 
 scan_success_file_exists() {
